#!/bin/ksh
# -----------------------------------------------------------------------
# Filename:   dbcopy_feeder.ksh
# Purpose:    Job scheduler - copy Oracle database tables between 
#             databases in parallel.
# Author:     Frank Naude (frank@ibi.co.za)
# Date:       08-Mar-2002
# -----------------------------------------------------------------------

# File defining schema owners and tables to copy
# Generate this list with script dbcopy_gentablist.sql
TABLES=dbcopy.tablist

# Set job concurrency 
# 4 per CPU is normally a good start - experimenet to get optimal value
MAXJOBS=16

# -----------------------------------------------------------------------
# Please do not change anything below this line
# -----------------------------------------------------------------------

# Create directory for detailed log files
if [ ! -d logs ]; then
   mkdir logs
fi

print "`date`: Start copying tables..." >dbcopy.log

# ---------------------------------------------------------------------
# Start sheduling jobs...
# ---------------------------------------------------------------------
print "Start sheduling COPY jobs..."
tabno=1

while true
do
  # Check number of jobs currently running
  JOBS=`ps -ef|grep dbcopy.ksh|grep $$|grep -v grep|wc -l`
  # ((JOBS=JOBS/2))
  echo "= Jobs currently running: $JOBS"
  if [ "${MAXJOBS}" -le "${JOBS}" ]; then
     print "% No empy slots for new jobs at this time..."
  else
     # Find next table to copy 
     PREVTAB=$TAB
     TABLINE=`tail -${tabno} ${TABLES} | head -1`
     OWNER=`echo $TABLINE | awk '{print $1}'`
     TAB=`echo $TABLINE | awk '{print $2}'`

     if [ "${TAB}" = "${PREVTAB}" ]; then
        print "Scheduling done. All $((tabno-1)) tables submitted. Waiting for jobs to complete..." >>dbcopy.log
        wait
        print "`date`: end copying..." >>dbcopy.log
        exit;
     fi
     print "+ Start job ${tabno} for $TAB..."
     nohup ./dbcopy.ksh $$ $OWNER $TAB &
     ((tabno=tabno+1))
  fi

  done
  sleep 3
done

# EOF
